Package: chipseq
Title: chipseq: A package for analyzing chipseq data
Version: 1.61.0
Authors@R: c(
        person("Deepayan", "Sarkar", role = "aut"),
        person("Robert", "Gentleman", role = "aut"),
        person("Michael", "Lawrence", role = "aut"),
        person("Zizhen", "Yao", role = "aut"),
        person("Oluwabukola", "Bamigbade", role = "ctb",
                comment = "Converted vignette from Sweave to R Markdown / HTML."),
        person("Bioconductor Package Maintainer",
                email = "maintainer@bioconductor.org", role = "cre"))
Description: Tools for helping process short read data for chipseq
        experiments.
Depends: R (>= 3.5.0), methods, BiocGenerics (>= 0.1.0), S4Vectors (>=
        0.17.25), IRanges (>= 2.13.12), GenomicRanges (>= 1.31.8),
        ShortRead
Imports: methods, stats, lattice, BiocGenerics, IRanges, GenomicRanges,
        ShortRead
Suggests: BSgenome, GenomicFeatures, TxDb.Mmusculus.UCSC.mm9.knownGene,
        BSgenome.Mmusculus.UCSC.mm9, BiocStyle, knitr
License: Artistic-2.0
LazyLoad: yes
biocViews: ChIPSeq, Sequencing, Coverage, QualityControl, DataImport
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/chipseq
git_branch: devel
git_last_commit: f81d061
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.23
Date/Publication: 2025-10-31
NeedsCompilation: yes
Packaged: 2025-10-31 21:05:31 UTC; biocbuild
Author: Deepayan Sarkar [aut],
  Robert Gentleman [aut],
  Michael Lawrence [aut],
  Zizhen Yao [aut],
  Oluwabukola Bamigbade [ctb] (Converted vignette from Sweave to R
    Markdown / HTML.),
  Bioconductor Package Maintainer [cre]
Maintainer: Bioconductor Package Maintainer <maintainer@bioconductor.org>
