useDynLib(chipseq)

## imports
import(BiocGenerics)
import(S4Vectors)
import(IRanges)

importClassesFrom(methods, list)
importClassesFrom(ShortRead, AlignedRead)

importMethodsFrom(methods, coerce)
importMethodsFrom(GenomicRanges, strand)
importMethodsFrom(ShortRead, name, readAligned)

importFrom(methods, as, is, new)
importFrom(lattice, panel.abline, panel.polygon, xyplot)
importFrom(ShortRead, alignQualityFilter, chromosome, chromosomeFilter,
           compose, occurrenceFilter, position, srFilter, strandFilter,
           uniqueFilter)
importFrom(stats, dnorm, dpois, ppois)


## context.R
## think these should be stuck into GenomicFeatures
##exportMethods(addNearestTss, addGenomicContext)

## diffpeaks.R.  Include diffPeakSummaryRef ?
export(diffPeakSummary, laneSubsample) 

## funs.R

export(chipseqFilter)

## intersect.R
## nothing right now

## islands.R
export(islandDepthPlot)

## plots.R
export(coverageplot)

## stats.R
export(sparse.density, basesCovered, densityCorr, estimate.mean.fraglen)
exportMethods(estimate.mean.fraglen)

## subsets.R
export(subsetSummary, peakCutoff)

## peaks.R
export(peakSummary)
